#!/usr/bin/env python
# -*- coding: utf-8 -*-

from Tkinter import *
import tkFileDialog
import tkMessageBox
import serial

import matplotlib
matplotlib.use('TkAgg')
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
from matplotlib.figure import Figure

import matplotlib.pyplot as window_plt

f = Figure(figsize=(5,4), dpi=100)
plt = f.add_subplot(111)
#plt.plot([0,1,2,3,4],[0,1,4,9,16])


#CONSTANT VALUE OF FOOTSTEP DISTANCE
FOOTSTEP = float(0.54)

def fix_arduino():
	s = serial.Serial(v.get(),9600, timeout=10)
	s.write("A")
	s.close()

def download_data():
	global data_log
	print "Downloading Data"
	s = serial.Serial(v.get(), 9600, timeout=10)
	s.write("R")
	data = s.readline()
	if data:
		data_log = []
	while data:
		data_log.append(data)
		data = s.readline()
	s.close()
	tkMessageBox.showinfo("Data Download", "Complete")
		
def save_log():
	print "Saving data log"
	filename = tkFileDialog.asksaveasfilename()
	f = open(filename, 'w')
	for i in range(0,len(data_log)):
		f.write(data_log[i])
		f.write("\n")
	f.close()
	tkMessageBox.showinfo("Data Save", "Complete")


def graph_data():
	global graph_choice
	clear_graph()
	
	print "graphing " + str(graph_choice.get()) 
	
	if (graph_choice.get() == 1):
		graph_distance()
	
	elif (graph_choice.get() == 2):
		graph_velocity()
		
	elif (graph_choice.get() == 3):
		graph_acceleration()
	
def graph_distance():
	global canvas
	global f
	global plt
	global distance_xvals
	global distance_yvals
	
	print "graphing"
	calc_distance()

		
	plt.plot(distance_xvals, distance_yvals)
	#f.xlabel('Time (Seconds)')
	#f.ylabel('Distance (meters)')
	canvas.show()
	root.update()
	
def clear_graph():
	global canvas
	global plt
	plt.clear()
	canvas.draw()
	
def window_graph():
	print "graphing in matplotlib window"
	x_vals = []
	x_vals.append(0)
	y_vals = []
	y_vals.append(0)
	time = 0
	distance = 0
	for i in data_log:
		if (int(i) == 0):
			print "breaking"
			break
		time = time + (float(i)/1000)
		distance = distance + FOOTSTEP
		x_vals.append(time)
		y_vals.append(distance)
		
	window_plt.plot(x_vals,y_vals)
	window_plt.xlabel('Time (Seconds)')
	window_plt.ylabel('Distance (meters)')
	window_plt.show()
	
def graph_in_window():
	print "GRAPHING in matplotlib window"
	global graph_choice
	
	temp_xvals = ""
	temp_yvals = ""

	y_label = ""
	
	print "GRAPHING " + str(graph_choice.get()) 
	
	if (graph_choice.get() == 1):
		#graph_distance()
		calc_distance()
		temp_xvals = distance_xvals
		temp_yvals = distance_yvals
		y_label = 'Distance (meters)'
	
	elif (graph_choice.get() == 2):
		#graph_velocity()
		calc_velocity()
		temp_xvals = velocity_xvals
		temp_yvals = velocity_yvals
		y_label = 'Speed (m/s)'
		
	elif (graph_choice.get() == 3):
		#graph_acceleration()	
		calc_acceleration()
		temp_xvals = acceleration_xvals
		temp_yvals = acceleration_yvals
		y_label = 'Acceleration (m/s^2)'

	window_plt.plot(temp_xvals, temp_yvals)
	window_plt.xlabel('Time (Seconds)')
	window_plt.ylabel(y_label)
	window_plt.show()
	
def calc_velocity():
	global velocity_yvals
	global velocity_xvals
	print "calculating velocity"
	velocity_xvals = []
	velocity_xvals.append(0)
	velocity_yvals = []
	velocity_yvals.append(0)
	time = 0
	distance = 0
	delta_time = 0
	velocity = 0
	for i in data_log:
		if (int(i) == 0):
			print "breaking"
			break
		time = time + (float(i)/1000)
		delta_time = float(i) / 1000
		delta_distance = FOOTSTEP
		velocity = delta_distance / delta_time
		velocity_xvals.append(time)
		velocity_yvals.append(velocity)
	
	#plt.plot(x_vals,y_vals)
	#f.xlabel('Time (Seconds)')
	#f.ylabel('Distance (meters)')
	#canvas.show()
	#root.update()	
	
def graph_velocity():
	global plt
	global canvas
	calc_velocity()
	plt.plot(velocity_xvals,velocity_yvals)
	canvas.show()
	root.update()

def calc_acceleration():	
	global velocity_yvals
	global velocity_xvals
	global acceleration_xvals
	global acceleration_yvals
	print "calculating acceleration"
	acceleration_xvals = []
	#acceleration_xvals.append(0)
	acceleration_yvals = []
	#acceleration_yvals.append(0)
	for i in range(0, len(velocity_xvals) -1):
		acceleration_xvals.append(velocity_xvals[i])
		acceleration_yvals.append((velocity_yvals[i+1] - velocity_yvals[i]) / (velocity_xvals[i+1] - velocity_xvals[i]))

def graph_acceleration():
	global plt
	global canvas
	calc_acceleration()
	plt.plot(acceleration_xvals, acceleration_yvals)
	canvas.show()
	root.update()
	
def calc_distance():
	global distance_xvals
	global distance_yvals
	distance_xvals = []
	distance_xvals.append(0)
	distance_yvals = []
	distance_yvals.append(0)
	time = 0
	distance = 0
	for i in data_log:
		if (int(i) == 0):
			print "breaking"
			break
		time = time + (float(i)/1000)
		distance = distance + FOOTSTEP
		distance_xvals.append(time)
		distance_yvals.append(distance)

def show_stats():
	calc_distance()
	d = distance_yvals[len(distance_yvals) - 1]
	t = distance_xvals[len(distance_yvals) - 1]
	v = d/t
	stats_string = "Distance: " + str(d) + "m \n" + "Average speed: " + str(v) + " m/s \n" + "Total Time: " + str(t) + " sec \n"
	tkMessageBox.showinfo("Arduino Shoe Desktop Client" , stats_string)

# Hold downloaded data
data_log = []

distance_yvals = []
distance_xvals = []

velocity_yvals = []
velocity_xvals = []

acceleration_yvals = []
acceleration_xvals = []

root = Tk()
root.geometry("500x600")
root.title("Arduino Shoe Desktop Client")
arduino_photo = PhotoImage(file="arduino.gif")
label = Label(image=arduino_photo)
label.image = arduino_photo
label.pack()

v = StringVar()
v.set("/dev/ttyACM0")
serial_entry = Entry(root, textvariable=v)
serial_entry.pack()

button_fix = Button(root, text="Fix Arduino", command=fix_arduino)
button_fix.pack()

button_download = Button(root, text="Download", command=download_data)
button_download.pack()

button_save = Button(root, text="Save Data Log", command=save_log)
button_save.pack()

graph_choice = IntVar()
Radiobutton(root, text="Distance", variable=graph_choice, value=1).pack()
Radiobutton(root, text="Speed", variable=graph_choice, value=2).pack()
Radiobutton(root, text="Acceleration", variable=graph_choice, value=3).pack()

button_graph = Button(root, text="Graph", command=graph_data)
button_graph.pack()

button_clear_graph = Button(root, text="Clear Graph", command=clear_graph)
button_clear_graph.pack()

button_graph_window = Button(root, text="Graph in another window", command=graph_in_window)
button_graph_window.pack()

button_show_stats = Button(root, text="Show statistics", command=show_stats)
button_show_stats.pack(side=RIGHT)

#button_graph_velocity = Button(root, text="Graph velocity", command=graph_velocity)
#button_graph_velocity.pack()

#button_graph_acceleration = Button(root, text="Graph acceleration", command=graph_acceleration)
#button_graph_acceleration.pack()

canvas = FigureCanvasTkAgg(f, root)
canvas.show()
canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=1)


root.mainloop()
